; get some more memory
(expand 1)

; some fake definitions for Common Lisp pseudo compatiblity
(setq symbol-function symbol-value)
(setq fboundp boundp)
(setq first car)
(setq second cadr)
(setq rest cdr)
(setq plus +)
(setq times *)
(setq litatom symbolp)

; some more cxr functions
(defun caddr (x) (car (cddr x)))
(defun cadddr (x) (cadr (cddr x)))

; (when test code...) - execute code when test is true
(defmacro when (test &rest code)
          `(cond (,test ,@code)))

; (unless test code...) - execute code unless test is true
(defmacro unless (test &rest code)
          `(cond ((not ,test) ,@code)))

; (makunbound sym) - make a symbol be unbound
(defun makunbound (sym) (setq sym '*unbound*) sym)

; (objectp expr) - object predicate
(defun objectp (x) (eq (type x) 'OBJ))

; (filep expr) - file predicate
(defun filep (x) (eq (type x) 'FPTR))

; (unintern sym) - remove a symbol from the oblist
(defun unintern (sym) (cond ((member sym *oblist*)
                             (setq *oblist* (delete sym *oblist*))
                             t)
                            (t nil)))

; (mapcan ...)
(defmacro mapcan (&rest args) `(apply #'nconc (mapcar ,@args)))

; (mapcon ...)
(defmacro mapcon (&rest args) `(apply #'nconc (maplist ,@args)))

; (spaces n [<sinc>]) - print n nr of spaces to <sink>
(defun spaces (n &optional sinc)
	(dotimes (i n)
		 (if sinc
		     (princ " " sinc)
		     (princ " "))))

; (save fun) - save a function definition to a file
(defun save (fun)
       (let* ((fname (strcat (symbol-name fun) ".lsp"))
              (fp (openo fname)))
             (cond (fp (print (cons (selectq (car (eval fun))
					     (lambda 'defun)
					     (nlambda 'defsub)
					     (macro 'defmacro)
					     (error "Bad definition: " fun))
				    (cons fun (cdr (eval fun))))
			      fp)
                       (close fp)
                       fname)
                   (t nil))))

; (makefile file) - save a package of functions, variables & properties
(defun makefile (FILE)
       (if (eq (type FILE) (quote SYM))
           (setq FILE (symbol-name FILE)))
       (let* ((fileNAME (strcat FILE ".lsp"))
              (fileFNS  (intern (strcat FILE "fns" )))
              (fileVARS (intern (strcat FILE "vars")))
              (FILE (openo fileNAME)))
             (if (not (and (boundp fileFNS)
                           (listp (eval fileFNS))))
                 (set fileFNS nil))
             (if (not (and (boundp fileVARS)
                           (listp (eval fileVARS))))
                 (set fileVARS nil))
             (makefile-VAR fileFNS FILE)
             (terpri FILE)
             (makefile-VAR fileVARS FILE)
             (makefile-FNS (eval fileFNS) FILE)
             (dolist (var (eval fileVARS))
                     (cond ((symbolp var)
                             (terpri FILE)
                             (makefile-VAR var FILE))
                           ((consp var)
                             (cond ((eq (car var) 'E)
                                     (dolist (sexpr (cdr var))
                                             (eval sexpr)))
                                   ((eq (car var) 'P)
                                     (dolist (sexpr (cdr var))
                                             (terpri FILE)
                                             (print sexpr FILE)))
                                   ((eq (car var) 'PROP)

                                     nil)
                                   ((eq (car var) 'FNS)
                                     (makefile-FNS (cdr var) FILE))
                                   ((eq (car var) 'VARS)
                                     (dolist (var-var (cdr var))
                                             (cond ((symbolp var-var)
                                                     (terpri FILE)
                                                     (makefile-VAR var-var
                                                                   FILE))
                                                   ((and (consp var-var)
                                                         (eql (length var-var)
                                                              2))
                                                     (terpri FILE)
                                                     (print (cons 'setq
                                                                  var-var)
                                                            FILE))
                                                   (t nil))))
                                   (t nil)))
                           (t nil)))
             (close FILE)
             fileNAME))

(defun makefile-VAR (var fp)
       (print (list 'setq var (list 'quote (eval var)))
              fp))

(defun makefile-FNS (funlist fp)
       (dolist (fun funlist)
               (terpri fp)
               (if (symbolp fun)
                   (print (cons (if (eq (car (eval fun))
                                        'lambda)
                                    'defun
                                    'defmacro)
                                (cons fun
                                      (cdr (eval fun))))
                          fp))))

; (debug) - enable debug breaks
(defun debug ()
       (setq *breakenable* t))

; (nodebug) - disable debug breaks
(defun nodebug ()
       (setq *breakenable* nil))

; initialize to enable breaks but no trace back
(setq *breakenable* t)
(setq *tracenable* nil)
